-------------------------------------------------------------------------------
-- Addon - Aggro - Init
-------------------------------------------------------------------------------
-- Version
local MAJOR = "Avion2_Aggro"
local MINOR = tonumber(("5"):match("%d+")) or 0

-- Dependencies
if not Avion2Core 				then error(MAJOR .. " requires Avion2.") end
if not LibStub("AceEvent-3.0")	then error(MAJOR .. " requires AceEvent-3.0.") end

-- Create
Avion2_Aggro 		= Avion2Core:NewModule(MAJOR, "AceEvent-3.0")
Avion2_Aggro.major 	= MAJOR
Avion2_Aggro.minor 	= MINOR


-------------------------------------------------------------------------------
-- Addon - Aggro - Variables
-------------------------------------------------------------------------------
-- Lib for aggro alert (should be replaced with WotLk)
local LibBanzai =  LibStub:GetLibrary("LibBanzai-2.0")

-- Translation
local L 		= LibStub("AceLocale-3.0"):GetLocale(MAJOR)
local coreL		= LibStub("AceLocale-3.0"):GetLocale(Avion2Core.major)

-- Local functions
local CombatEnter, CombatLeave, UpdateAggro


-------------------------------------------------------------------------------
-- Addon - Aggro - Register with core
-------------------------------------------------------------------------------
-- Register SavedVariables
function Avion2_Aggro:RegisterModuleDB()
	-- Register own saved variables DB
	self.db = Avion2Core.db:RegisterNamespace(Avion2_Aggro.major, {
		profile = {
			RaidGainedAggro 	= { 
				animationList 		= { },
				ignoreTanks			= true,
				cancelAnimation		= 0,
				enabled				= false,
			},
			PlayerGainedAggro = { 
				animationList 		= { },
				onlyIfNobodyElse 	= false,
				cancelAnimation		= 0,
				enabled				= false,
			},
			PlayerLostAggro = { 
				animationList 		= { },
				onlyIfSomeoneElse 	= false,
				cancelAnimation		= 0,
				enabled 			= false,
			},
			CombatEnter 	= {
				animationList 		= { },
				cancelAnimation		= 0,
				enabled 			= false,
			},
			lastMinor 		= MINOR,
		},
	})
	
	-- Updateto to new rev, data converters should go here
	self.db.profile.lastMinor = Avion2_Aggro.minor
end


-- (Un)register (un)needed events
function Avion2_Aggro:UpdateEventRegistration()
	-- Aggro events
	if (self:IsEnabled() and (
		self.db.profile.RaidGainedAggro.enabled or
		self.db.profile.PlayerGainedAggro.enabled or 
		self.db.profile.PlayerLostAggro.enabled
	)) then
		LibBanzai:RegisterCallback(UpdateAggro)
	else
		LibBanzai:RegisterCallback(UpdateAggro)
		LibBanzai:UnregisterCallback(UpdateAggro)
	end
	
	-- Cambat enter event
	if (self:IsEnabled() and self.db.profile.CombatEnter.enabled) then
		self:RegisterEvent("PLAYER_ENTER_COMBAT", CombatEnter)
		self:RegisterEvent("PLAYER_REGEN_DISABLED", CombatEnter)
		self:RegisterEvent("PLAYER_LEAVE_COMBAT", CombatLeave)
		self:RegisterEvent("PLAYER_REGEN_ENABLED", CombatLeave)
	else
		self:UnregisterEvent("PLAYER_ENTER_COMBAT")
		self:UnregisterEvent("PLAYER_REGEN_DISABLED")
		self:UnregisterEvent("PLAYER_LEAVE_COMBAT")
		self:UnregisterEvent("PLAYER_REGEN_ENABLED")
	end
end


-- Update animation (convert animationFunc: string name to function pointer)
function Avion2_Aggro:LoadAnimationTables()
	Avion2Core:LoadAnimationTables(self.db.profile.RaidGainedAggro.animationList)
	Avion2Core:LoadAnimationTables(self.db.profile.PlayerGainedAggro.animationList)
	Avion2Core:LoadAnimationTables(self.db.profile.PlayerLostAggro.animationList)
	Avion2Core:LoadAnimationTables(self.db.profile.CombatEnter.animationList)
end


-------------------------------------------------------------------------------
-- Addon - Aggro - Options
-------------------------------------------------------------------------------
-- Fetch animationTable from animationID (as suplied by eg. Menu_Reuqest)
function Avion2_Aggro:GetAnimationTable(animationID)
	success, _, arg1, arg2 = self:Deserialize(animationID)
	
	if (success) then
		return self.db.profile[arg1].animationList[arg2]
	else
		return false
	end
end


-- Build Optionstable
function Avion2_Aggro:UpdateOptionTable(args)
	-- Update options when enabled and initialized
	if (self.db and self:IsEnabled()) then
		local serializedInfo
		
		-- Options for raid member gained aggro
		args[self.major .. "_RaidGainedAggro"] = {
			type = 'group',
			name = L["RaidGainedAggro"],
			desc = L["RaidGainedAggro_Desc"],
			args = { 
				list = {
					order = 1,
					type = 'group',
					name = coreL["AnimationList"],
					desc = coreL["AnimationList_Desc"],
					args = {
						new = Avion2Core:GetDefaultNewAnimationOption(self.db.profile.RaidGainedAggro.animationList),
					},
				},
				ignoreTanks = {
					order = 2,
					type = 'toggle',
					name = L["IgnoreTanks"],
					desc = L["IgnoreTanks_Desc"],
					get = function()
						return self.db.profile.RaidGainedAggro.ignoreTanks
					end,
					set = function()
						self.db.profile.RaidGainedAggro.ignoreTanks = not self.db.profile.RaidGainedAggro.ignoreTanks
					end,
				},
				cancelAnimation = {
					order = 3,
					type = 'select',
					name = coreL["CancelAnimation"],
					desc = coreL["CancelAnimation_Desc"],
					values = {
						[0] = coreL["DontCancel"],
						[1] = coreL["FadeOut"],
						[2] = coreL["Cancel"],
					},
					get = function()
						return self.db.profile.RaidGainedAggro.cancelAnimation
					end,
					set = function(info, val)
						self.db.profile.RaidGainedAggro.cancelAnimation = val
					end,
				},
				enabled = {
					order = 4,
					type = 'toggle',
					name = coreL["Enabled"],
					desc = coreL["Enabled_Desc"],
					get = function()
						return self.db.profile.RaidGainedAggro.enabled
					end,
					set = function()
						self.db.profile.RaidGainedAggro.enabled = not self.db.profile.RaidGainedAggro.enabled
						
						self:UpdateEventRegistration()
					end,
				},
			},
		}
		-- Add options for each animation
		for animationListID, _ in pairs(self.db.profile.RaidGainedAggro.animationList) do
			-- Combine all info to find this animation in module db in ONE string
			serializedInfo = self:Serialize(Avion2_Aggro.major, "RaidGainedAggro", animationListID)
			
			-- Update option table
			args[self.major .. "_RaidGainedAggro"].args.list.args[string.format("Animation_%d", animationListID)] = Avion2Core:GetDefaultAnimationOption(self.db.profile.RaidGainedAggro.animationList, animationListID, serializedInfo)
		end

		-- Options for player aggro gain
		args[self.major .. "_PlayerGainedAggro"] = {
			type = 'group',
			name = L["PlayerGainedAggro"],
			desc = L["PlayerGainedAggro_Desc"],
			args = { 
				list = {
					order = 1,
					type = 'group',
					name = coreL["AnimationList"],
					desc = coreL["AnimationList_Desc"],
					args = {
						new = Avion2Core:GetDefaultNewAnimationOption(self.db.profile.PlayerGainedAggro.animationList),
					},
				},
				onlyIfNobodyElse = {
					order = 2,
					type = 'toggle',
					name = L["OnlyIfNobodyElse"],
					desc = L["OnlyIfNobodyElse_Desc"],
					get = function()
						return self.db.profile.PlayerGainedAggro.onlyIfNobodyElse
					end,
					set = function()
						self.db.profile.PlayerGainedAggro.onlyIfNobodyElse = not self.db.profile.PlayerGainedAggro.onlyIfNobodyElse
					end,
				},
				cancelAnimation = {
					order = 2,
					type = 'select',
					name = coreL["CancelAnimation"],
					desc = coreL["CancelAnimation_Desc"],
					values = {
						[0] = coreL["DontCancel"],
						[1] = coreL["FadeOut"],
						[2] = coreL["Cancel"],
					},
					get = function()
						return self.db.profile.PlayerGainedAggro.cancelAnimation
					end,
					set = function(info, val)
						self.db.profile.PlayerGainedAggro.cancelAnimation = val
					end,
				},
				enabled = {
					order = 4,
					type = 'toggle',
					name = coreL["Enabled"],
					desc = coreL["Enabled_Desc"],
					get = function()
						return self.db.profile.PlayerGainedAggro.enabled
					end,
					set = function()
						self.db.profile.PlayerGainedAggro.enabled = not self.db.profile.PlayerGainedAggro.enabled
						
						self:UpdateEventRegistration()
					end,
				},
			},
		}
		-- Add options for each animation
		for animationListID, _ in pairs(self.db.profile.PlayerGainedAggro.animationList) do
			-- Combine all info to find this animation in module db in ONE string
			serializedInfo = self:Serialize(Avion2_Aggro.major, "PlayerGainedAggro", animationListID)
			
			-- Update option table
			args[self.major .. "_PlayerGainedAggro"].args.list.args[string.format("Animation_%d", animationListID)] = Avion2Core:GetDefaultAnimationOption(self.db.profile.PlayerGainedAggro.animationList, animationListID, serializedInfo)
		end
		
		-- Options for player aggro gain
		args[self.major .. "_PlayerLostAggro"] = {
			type = 'group',
			name = L["PlayerLostAggro"],
			desc = L["PlayerLostAggro_Desc"],
			args = { 
				list = {
					order = 1,
					type = 'group',
					name = coreL["AnimationList"],
					desc = coreL["AnimationList_Desc"],
					args = {
						new = Avion2Core:GetDefaultNewAnimationOption(self.db.profile.PlayerLostAggro.animationList),
					},
				},
				onlyIfSomeoneElse = {
					order = 2,
					type = 'toggle',
					name = L["OnlyIfSomeoneElse"],
					desc = L["OnlyIfSomeoneElse_Desc"],
					get = function()
						return self.db.profile.PlayerLostAggro.onlyIfSomeoneElse
					end,
					set = function()
						self.db.profile.PlayerLostAggro.onlyIfSomeoneElse = not self.db.profile.PlayerLostAggro.onlyIfSomeoneElse
					end,
				},
				cancelAnimation = {
					order = 2,
					type = 'select',
					name = coreL["CancelAnimation"],
					desc = coreL["CancelAnimation_Desc"],
					values = {
						[0] = coreL["DontCancel"],
						[1] = coreL["FadeOut"],
						[2] = coreL["Cancel"],
					},
					get = function()
						return self.db.profile.PlayerLostAggro.cancelAnimation
					end,
					set = function(info, val)
						self.db.profile.PlayerLostAggro.cancelAnimation = val
					end,
				},
				enabled = {
					order = 4,
					type = 'toggle',
					name = coreL["Enabled"],
					desc = coreL["Enabled_Desc"],
					get = function()
						return self.db.profile.PlayerLostAggro.enabled
					end,
					set = function()
						self.db.profile.PlayerLostAggro.enabled = not self.db.profile.PlayerLostAggro.enabled
						
						self:UpdateEventRegistration()
					end,
				},
			},
		}
		-- Add options for each animation
		for animationListID, _ in pairs(self.db.profile.PlayerLostAggro.animationList) do
			-- Combine all info to find this animation in module db in ONE string
			serializedInfo = self:Serialize(Avion2_Aggro.major, "PlayerLostAggro", animationListID)
			
			-- Update option table
			args[self.major .. "_PlayerLostAggro"].args.list.args[string.format("Animation_%d", animationListID)] = Avion2Core:GetDefaultAnimationOption(self.db.profile.PlayerLostAggro.animationList, animationListID, serializedInfo)
		end
		
		-- Options for entering combat
		args[self.major .. "_CombatEnter"] = {
			type = 'group',
			name = L["CombatEnter"],
			desc = L["CombatEnter_Desc"],
			args = { 
				list = {
					order = 1,
					type = 'group',
					name = coreL["AnimationList"],
					desc = coreL["AnimationList_Desc"],
					args = {
						new = Avion2Core:GetDefaultNewAnimationOption(self.db.profile.CombatEnter.animationList),
					},
				},
				cancelAnimation = {
					order = 2,
					type = 'select',
					name = coreL["CancelAnimation"],
					desc = coreL["CancelAnimation_Desc"],
					values = {
						[0] = coreL["DontCancel"],
						[1] = coreL["FadeOut"],
						[2] = coreL["Cancel"],
					},
					get = function()
						return self.db.profile.CombatEnter.cancelAnimation
					end,
					set = function(info, val)
						self.db.profile.CombatEnter.cancelAnimation = val
					end,
				},
				enabled = {
					order = 3,
					type = 'toggle',
					name = coreL["Enabled"],
					desc = coreL["Enabled_Desc"],
					get = function()
						return self.db.profile.CombatEnter.enabled
					end,
					set = function()
						self.db.profile.CombatEnter.enabled = not self.db.profile.CombatEnter.enabled
						
						self:UpdateEventRegistration()
					end,
				},
			},
		}
		-- Add options for each animation
		for animationListID, _ in pairs(self.db.profile.CombatEnter.animationList) do
			-- Combine all info to find this animation in module db in ONE string
			serializedInfo = self:Serialize(Avion2_Aggro.major, "CombatEnter", animationListID)
			
			-- Update option table
			args[self.major .. "_CombatEnter"].args.list.args[string.format("Animation_%d", animationListID)] = Avion2Core:GetDefaultAnimationOption(self.db.profile.CombatEnter.animationList, animationListID, serializedInfo)
		end
	else
		args[self.major .. "_RaidGainedAggro"]		= nil
		args[self.major .. "_PlayerGainedAggro"]	= nil
		args[self.major .. "_PlayerLostAggro"] 		= nil
		args[self.major .. "_CombatEnter"] 			= nil
	end
end


-------------------------------------------------------------------------------
-- Addon - Aggro - Events
-------------------------------------------------------------------------------
-- Player entered combat
CombatEnter = function ()
	Avion2Core:TriggerAnimationList("Avion2_CombatEnter", Avion2_Aggro.db.profile.CombatEnter.animationList)
end


-- Player entered combat
CombatLeave = function ()
	if (Avion2_Aggro.db.profile.CombatEnter.cancelAnimation ~= 0) then
		Avion2Core:StopAnimationList("Avion2_CombatEnter", Avion2_Aggro.db.profile.CombatEnter.animationList, (Avion2_Aggro.db.profile.CombatEnter.cancelAnimation == 1))
	end
end


-- Unit gained/lost aggro, trigger animation
UpdateAggro = function (aggro, unitName)
	-- Trigger for self
	if (unitName == UnitName("player")) then
		-- Aggro lost
		if (aggro == 0) then
			local someBodyElse = true
			
			-- Scan if aggro lost 
			if (Avion2_Aggro.db.profile.PlayerLostAggro.onlyIfSomeoneElse) then
				someBodyElse = false
				
				-- Scan if someone in raid has aggro (would be easier if LibBanzai:AggroCount() would exist :P)
				if (UnitInRaid("player")) then
					for member = 1, GetNumRaidMembers(), 1 do
						if (LibBanzai:GetUnitAggroByUnitId(string.format("raid%d", member)) and UnitName("player") ~= UnitName(string.format("raid%d", member))) then
							someBodyElse = true
							
							break
						end
					end
					
				-- Scan if someone in party has aggro(would be easier if LibBanzai:AggroCount() would exist :P)
				elseif (UnitInParty("player")) then
					for member = 1, GetNumPartyMembers(), 1 do
						if (LibBanzai:GetUnitAggroByUnitId(string.format("party%d", member)) and UnitName("player") ~= UnitName(string.format("party%d", member))) then
							someBodyElse = true
							
							break
						end
					end
				end
			end
			
			-- Somebody has aggro
			if (Avion2_Aggro.db.profile.PlayerLostAggro.enabled and someBodyElse) then
				Avion2Core:TriggerAnimationList("Avion2_PlayerLostAggro", Avion2_Aggro.db.profile.PlayerLostAggro.animationList)
			end
		
		-- Aggro lost
		elseif (aggro == 1) then
			local nobodyElse = true
			
			-- Scan raid/party if nobody has aggro
			if (Avion2_Aggro.db.profile.PlayerGainedAggro.onlyIfNobodyElse) then
				-- Scan if someone in raid has aggro
				if (UnitInRaid("player")) then
					for member = 1, GetNumRaidMembers(), 1 do
						if (LibBanzai:GetUnitAggroByUnitId(string.format("raid%d", member)) and UnitName("player") ~= UnitName(string.format("raid%d", member))) then
							nobodyElse = false
							
							break
						end
					end
					
				-- Scan if someone in party has aggro
				elseif (UnitInParty("player")) then
					for member = 1, GetNumPartyMembers(), 1 do
						if (LibBanzai:GetUnitAggroByUnitId(string.format("party%d", member)) and UnitName("player") ~= UnitName(string.format("party%d", member))) then
							nobodyElse = false
							
							break
						end
					end
				end
			end
			
			-- Nobody has aggro
			if (Avion2_Aggro.db.profile.PlayerGainedAggro.enabled and nobodyElse) then
				Avion2Core:TriggerAnimationList("Avion2_PlayerGainedAggro", Avion2_Aggro.db.profile.PlayerGainedAggro.animationList)
			end
		end
		
	-- Trigger for raid gained aggro
	elseif (aggro == 1 and Avion2_Aggro.db.profile.RaidGainedAggro.enabled) then
		-- Ignore tanks if wanted
		if (not Avion2_Aggro.db.profile.RaidGainedAggro.ignoreTanks or not (
		GetPartyAssignment("MAINASSIST", 	unitName, true) or
		GetPartyAssignment("MAINTANK", 		unitName, true) or
		oRa and oRA.db.profile.maintanktable[unitName])
		) then
			Avion2Core:TriggerAnimationList("Avion2_RaidGainedAggro", Avion2_Aggro.db.profile.RaidGainedAggro.animationList)
		end
	end
	
	-- Stop running animations
	if (unitName == UnitName("player")) then
		if (aggro == 0 and Avion2_Aggro.db.profile.PlayerGainedAggro.cancelAnimation ~= 0) then
			Avion2Core:StopAnimationList("Avion2_PlayerGainedAggro", Avion2_Aggro.db.profile.PlayerGainedAggro.animationList, (Avion2_Aggro.db.profile.PlayerGainedAggro.cancelAnimation == 1))
		elseif (aggro == 1 and Avion2_Aggro.db.profile.PlayerLostAggro.cancelAnimation ~= 0) then
			Avion2Core:StopAnimationList("Avion2_PlayerLostAggro", Avion2_Aggro.db.profile.PlayerLostAggro.animationList, (Avion2_Aggro.db.profile.PlayerLostAggro.cancelAnimation == 1))
		end
	elseif (aggro == 0 and Avion2_Aggro.db.profile.RaidGainedAggro.cancelAnimation ~= 0) then
		Avion2Core:StopAnimationList("Avion2_RaidGainedAggro", Avion2_Aggro.db.profile.RaidGainedAggro.animationList, (Avion2_Aggro.db.profile.RaidGainedAggro.cancelAnimation == 1))
	end
end
